import java.util.*;

public class TestTreeIterator {

  private static void makeSureAssertionsAreEnabled () {
    try {
      assert(false);
      System.out.println("Vous devez donner l'option -ea dans VM Arguments!");
      System.exit(1);
    } catch (AssertionError e) {}
  }

  public static void main(String[] args) {
    makeSureAssertionsAreEnabled();

    Tree<Integer> t =
      new Tree<Integer>(2,
        new Tree<Integer>(1, null, null),
        new Tree<Integer>(3, null, null)
      );
    System.out.println("This should print:");
    System.out.println("1 2 3 ");
    printIterable(t);

    t =
      new Tree<Integer>(4,
        new Tree<Integer>(1, null,
          new Tree<Integer>(2, null, new Tree<Integer>(3,null,null))
        ),
        new Tree<Integer>(8,
          new Tree<Integer>(6,
            new Tree<Integer>(5,null,null),
            new Tree<Integer>(7,null,null)
          ),
          null
        )
      );
    System.out.println("This should print:");
    System.out.println("1 2 3 4 5 6 7 8 ");
    printIterable(t);

    t = randomBST(15);
    printTree(t); System.out.println();
    System.out.println("This should print:");
    System.out.println("0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 ");
    printIterable(t);

    // Bonus question: pre-order iterators.
    t = randomPrefix(15);
    // Replace the next line with a method call that constructs a pre-order iterator.
    Iterator<Integer> it = null; // t.preorderIterator();
    if (it != null) {
      printTree(t); System.out.println();
      System.out.println("This should print:");
      System.out.println("0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 ");
      printIterator(it);
    }
  }

  // toutes les méthodes ci-dessous ne sont là que pour les tests
  static Tree<Integer> bstAdd(Tree<Integer> t, int x) {
    if (t == null) return new Tree<Integer>(x, null, null);
    if (x < t.val) return new Tree<Integer>(t.val, bstAdd(t.left, x), t.right);
    else return new Tree<Integer>(t.val, t.left, bstAdd(t.right, x));
  }

  static void swap(int[] a, int i, int j) {
    int tmp = a[i];
    a[i] = a[j];
    a[j] = tmp;
  }

  static Tree<Integer> randomBST(int n) {
    int[] a = new int[n];
    for (int i = 0; i < n; i++) a[i] = i;
    for (int i = 1; i < n; i++) {
      int j = (int)(Math.random() * (i + 1));
      swap(a, i, j);
    }
    Tree<Integer> t = null;
    for (int x: a) t = bstAdd(t, x);
    return t;
  }

  static Tree<Integer> randomPrefix(int lo, int hi) {
    if (hi <= lo) return null;
    int left = (int)(Math.random() * (hi - lo));
    return new Tree<Integer>(lo,
                             randomPrefix(lo + 1, lo + left + 1),
                             randomPrefix(lo + left + 1, hi));
  }

  static Tree<Integer> randomPrefix(int n) {
    return randomPrefix(0, n);
  }

  static void printTree(Tree<Integer> t) {
    if (t == null) return;
    System.out.print("(");
    printTree(t.left);
    System.out.print(t.val);
    printTree(t.right);
    System.out.print(")");
  }

  static void printIterator (Iterator<Integer> it) {
    while (it.hasNext())
      System.out.print(it.next() + " ");
    System.out.println();
  }

  static void printIterable (Iterable<Integer> c) {
    printIterator(c.iterator());
  }

}
